'''Shorty Forms'''
from django import forms
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

from .models import ShortURL
from .app_settings import EXTERNAL_FLAG


class ShortURLForm(forms.ModelForm):
    redirect = forms.URLField(required=True, widget=forms.URLInput(attrs={'placeholder': 'Enter a URL'}))
    path = forms.SlugField(required=False, widget=forms.TextInput(attrs={'placeholder': 'short-url'}))
    override_existing = forms.CharField(required=False, widget=forms.HiddenInput())

    def __init__(self, request, *args, **kwargs):
        self.request = request
        return super(ShortURLForm, self).__init__(*args, **kwargs)

    def clean_override_existing(self):
        redirect = self.cleaned_data['redirect']
        override_existing = self.cleaned_data['override_existing']

        if override_existing != '1':
            short_urls = ShortURL.objects.filter(redirect=redirect)
            if short_urls:
                self.request.previous_short_urls = short_urls
                raise forms.ValidationError('That URL has been shortened previously.')

        return override_existing

    class Meta:
        model = ShortURL
        fields = ['redirect', 'path']
        if EXTERNAL_FLAG:
            fields.append('external')



'''Shorty App Settings'''
from django.conf import settings


ADMIN_ENABLED = getattr(settings, 'SHORTY_ADMIN_ENABLED', True)

EXTERNAL_FLAG = getattr(settings, 'SHORTY_EXTERNAL_FLAG', False)

CANONICAL_DOMAIN = getattr(settings, 'SHORTY_CANONICAL_DOMAIN', None)



'''Shorty URLs'''
from django.conf.urls import url

from . import views
from . import app_settings

urlpatterns = [
    url(r'^$', views.do_redirect, name='redirect_base'),
    url(r'^(?P<slug>[-_\w]+)/?$', views.do_redirect, name='redirect'),
]

if app_settings.ADMIN_ENABLED:
    urlpatterns = [
        url(r'^admin/$', views.home, name='home'),
        url(r'^admin/delete/$', views.delete, name='delete'),
    ] + urlpatterns



'''Shorty Models'''
from django.db import models
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

import string
import random


RANDOM_SLUG_CHOICES = string.ascii_letters + string.digits + '_-'


def random_slug():
    while True:
        slug = ''.join(random.choice(RANDOM_SLUG_CHOICES) for _ in range(7))
        try:
            ShortURL.objects.get(path=slug)
        except ShortURL.DoesNotExist:
            return slug


class ShortURL(models.Model):
    path = models.SlugField(unique=True, error_messages={'unique': 'This short URL is already in use; please choose something different, or leave blank for a random URL',})
    redirect = models.URLField()
    user = models.ForeignKey(User, related_name='short_urls')
    created = models.DateTimeField(auto_now_add=True)
    external = models.BooleanField(default=False, help_text='Should this short link be available outside the company\'s network?')

    class Meta:
        ordering = ('-created', )
        verbose_name = 'Short URL'
        verbose_name_plural = 'Short URLs'

    def __unicode__(self):
        return '{}: {}'.format(self.path, self.redirect)

    def save(self, *args, **kwargs):
        if not self.path:
            self.path = random_slug()
        return super(ShortURL, self).save(*args, **kwargs)

    def build_uri(self, request):
        return request.build_absolute_uri(reverse('redirect', kwargs={'slug': self.path}))



from setuptools import find_packages, setup

from shorty.version import __VERSION__

dependencies=[
    'django',
    'django-autoconfig >= 0.5.0',
    'django-nuit >= 1.0.0, < 2.0.0',
]
test_dependencies=[
    'django-setuptest',
    'mock',
]

setup(
    name='djshorty',
    version=__VERSION__,
    description='A Django URL shortening app',
    author='Ben Cardy',
    author_email='ben.cardy@ocado.com',
    packages=find_packages(),
    install_requires=dependencies,
    # To run tests via python setup.py test
    tests_require=test_dependencies,
    test_suite='setuptest.setuptest.SetupTestSuite',
    include_package_data=True,
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
    ],
)



DEFAULT_SETTINGS = {
    'AUTOCONFIG_URL_PREFIXES': {
        'shorty': '',
    },
}

SETTINGS = {
    'INSTALLED_APPS': [
        'django.contrib.admin',
        'nuit',
    ],
    'NUIT_GLOBAL_TITLE': 'Shorty',
}
